// PVCSMerge.cpp : Defines the entry point for the console application.
//

// Instructions:
// 1. Create an empty console application called pvcsmerge
// 2. Add this file to the project
// 3. Enable or disable the _FORCE_RULES precompiler definition as required
//    If _FORCE_RULES is to be used, uncomment the rules (see: ForceRules) to be used.
// 4. Build the project (in release)
// 5. Copy the executable to {Program Files}\PVCS\vm\win32\bin
//    Make a backup of the original if unsure about this
// DONE!

#include <stdio.h>
#include <fstream>	    // STL
#include <process.h>    // spawn
#include <string>       // STL
#include <windows.h>    // registry access

#define _FORCE_RULES     // uncomment this line for remove forced rules (see: ForceRules)

static const char* Quote = "\"";
static const char* File1 = "File 1:";
static const char* File2 = "File 2:";
static const char  Space = ' ';

static const char* BC2Key     = "Software\\Scooter Software\\Beyond Compare";

// only active when _FORCE_RULES defined (uncomment only one)
static const char* ForceRules = " /rules=\"C/C++/C# Source\"";
//static const char* ForceRules = " /rules=\"Comma Separated Values\"";
//static const char* ForceRules = " /rules=\"Delphi Source\"";
//static const char* ForceRules = " /rules=\"Delphi Forms\"";
//static const char* ForceRules = " /rules=\"General Text\"";
//static const char* ForceRules = " /rules=\"HTML\"";
//static const char* ForceRules = " /rules=\"Java\"";
//static const char* ForceRules = " /rules=\"Python\"";
//static const char* ForceRules = " /rules=\"Registry Dump\"";
//static const char* ForceRules = " /rules=\"UTF8 Text\"";
//static const char* ForceRules = " /rules=\"Visual Basic\"";
//static const char* ForceRules = " /rules=\"XML\"";

struct FileInfo
{
    std::string strFilename;
    std::string strDescription;

    enum { BufSize = 255 };

    char* GetDescription(const int nId)
    {
        static char pszDesc[BufSize];
        if (strDescription.empty())
        {
            return NULL;
        }
        else if (strDescription == strFilename)
        {
            return NULL;
        }
        else
        {
            ::sprintf(pszDesc, "/title%d=\"%s\"", nId, strDescription.c_str());
        }

        return pszDesc;
    }
};

int main(int argc, char* argv[])
{
#ifdef _DEBUG
    std::ofstream fout;
    fout.open("c:\\pvcsmerge.txt");
    if (fout.is_open())
    {
        for (int i=0; i<argc; i++)
        {
            fout << i << " = " << argv[i] << std::endl;
        }
    }
#endif // _DEBUG

    const char* pszFilename = argv[2];
    if (pszFilename == NULL)
    {
#ifdef _DEBUG
        fout << "Error: not enough parameters" << std::endl;
        fout.close();
#endif // _DEBUG
        return -1;                      // not enough parameters, bail out
    }

	std::ifstream fin;
    fin.open(pszFilename);
    if (fin.is_open())
    {
        const int bufsize = 255;
        char buffer[bufsize];

        FileInfo left;
        FileInfo right;

        while (!fin.eof())
        {
            fin.getline(buffer, bufsize);

#ifdef _DEBUG
            if (fout.is_open())
            {
                fout << buffer << std::endl;
            }
#endif // _DEBUG

            if (::strstr(buffer, "ANCESTOR") == buffer)
            {
                // found file 1
                char* pFileStart  = ::strstr(buffer, Quote) + 1;
                char* pFileEnd    = ::strstr(pFileStart, Quote);

                if (pFileEnd)
                {
                    char* pNameStart  = ::strstr(pFileEnd + 1, File1) + ::strlen(File1);
                    while (*pNameStart == Space)
                        pNameStart++;

                    char* pNameEnd = ::strstr(pNameStart, Quote);

                    if (pNameEnd)
                        *pNameEnd = 0x0;

                    *pFileEnd = 0x0;

                    left.strDescription = pNameStart;
                }

                left.strFilename = pFileStart;
            }
            else if (::strstr(buffer, "DERIVATIVES") == buffer)
            {
                // found file 2
                char* pFileStart  = ::strstr(buffer, Quote) + 1;
                char* pFileEnd    = ::strstr(pFileStart, Quote);

                if (pFileEnd)
                {
                    char* pNameStart  = ::strstr(pFileEnd + 1, File2) + ::strlen(File2);
                    while (*pNameStart == Space)
                        pNameStart++;

                    char* pNameEnd = ::strstr(pNameStart, Quote);

                    if (pNameEnd)
                        *pNameEnd = 0x0;

                    *pFileEnd = 0x0;

                    right.strDescription = pNameStart;
                }

                right.strFilename = pFileStart;
            }
        }
        fin.close();


        std::string strArgList;
        strArgList.reserve(255);

        char* pszDesc = left.GetDescription(1);
        if (pszDesc > NULL)
        {
            strArgList.append(pszDesc);
            strArgList.append(" /leftreadonly ");
        }

        pszDesc = right.GetDescription(2);
        if (pszDesc > NULL)
        {
            strArgList.append(pszDesc);
            strArgList.append(" /rightreadonly ");
        }

        strArgList.append("\"");
        strArgList.append(left.strFilename.c_str());
        strArgList.append("\" \"");
        strArgList.append(right.strFilename.c_str());
        strArgList.append("\"");

#ifdef _FORCE_RULES
#pragma message("Build info: forcing rules")
        strArgList.append(ForceRules);
#endif // _FORCE_RULES

#ifdef _DEBUG
        if (fout.is_open())
        {
            fout << "argument list:" << std::endl;
            fout << strArgList.c_str() << std::endl;
        }
#endif // _DEBUG

        char pszBC2[_MAX_PATH];
        ::memset(pszBC2, 0x0, _MAX_PATH);

        HKEY hKeyBC = NULL;
        LONG lResult = ::RegOpenKeyEx(HKEY_CURRENT_USER, BC2Key, 0, KEY_READ, &hKeyBC);
        if (lResult == ERROR_SUCCESS)
        {
            DWORD dwType = REG_SZ;
            DWORD dwBufSize = _MAX_PATH;
            lResult = ::RegQueryValueEx(hKeyBC, "ExePath", NULL, &dwType, (byte*)pszBC2, &dwBufSize);

#ifdef _DEBUG
            if (fout.is_open())
            {
                fout << "launching BC2 from " << pszBC2 << std::endl;
            }
#endif // _DEBUG

            ::RegCloseKey(hKeyBC);
        }

        if (lResult == ERROR_SUCCESS)
        {
            ::_spawnl(_P_WAIT, pszBC2, "/fv ", strArgList.c_str(), NULL);
        }
        else
        {
            // try default location
#ifdef _DEBUG
            if (fout.is_open())
            {
                fout << "launching BC2 from installation default location"<< std::endl;
            }
#endif // _DEBUG

            ::_spawnl(_P_WAIT, "C:\\Program Files\\Beyond Compare 2\\BC2.exe", "/fv ", strArgList.c_str(), NULL);
        }

    }
#ifdef _DEBUG
    else
    {
        fout << "Error: File not found (" << pszFilename << ")" << std::endl;
    }
    fout.close();
#endif // _DEBUG

    return 0;
}
